/* allegro_5.c
 * Ukazka programu v Allegru 
 * prikladek na mys a klavesnici
 *
 * Bernard Lidicky - 2004
 *
 * Je pod licenci GPL
 *
 * pro preklad mi fungovalo: "gcc allegro_5.c -o allegro_5 `allegro-config --cflags --libs`"
 */
 
#include <stdio.h>
#include <allegro.h>

// nastaveni rozliseni 
// V normalni hre by se melo nacotat z konfiguracniho souboru
#define SCREEN_X   640
#define SCREEN_Y   480
#define SCREEN_BPP  16


// druha bitmapa velikosti obrazovky
BITMAP *back = NULL;


// veci pro hru

// obrazek ryby
BITMAP *ryba;

// souradnice ryby
int x = 0, y = 0;


// barva podkladu
float r, g, b;    // barva rgb
float dr, dg, db; // zmena barvy

/*
 * Provede inicializaci knihovny, nastavi rozliseni
 */
void InitSystem()
{

	// Inicializace Allegra
	if ( allegro_init() < 0 ) {
		fprintf(stderr, "Selhala inicializace Allegra.\n");
		exit(1);
	}

	// nastaveni barevne hloubky
	set_color_depth(SCREEN_BPP);
	
	// nastaveni grafickeho modu
//	if ( set_gfx_mode(GFX_AUTODETECT, SCREEN_X, SCREEN_Y, 0, 0) != 0 ){
	if ( set_gfx_mode(GFX_XWINDOWS, SCREEN_X, SCREEN_Y, 0, 0) != 0 ){
			fprintf(stderr, "Nepodarilo nastavit rozliseni %dx%dx%d\n", SCREEN_X, SCREEN_Y, SCREEN_BPP);
			exit(1);
	}
}

/*
 * Provede na konci programu uklid
 */
void CleanAll()
{

	// uvolneni nactene bitmapky
	destroy_bitmap(ryba);

	// uvolneni bitmapy s druhou obrazovkou
	destroy_bitmap(back);
	
	
	// ukonceni Allegra - neni nutne volat
	allegro_exit();
	
}

/*
 * Nacte bitmapy a podobne veci
 */
void InitGame()
{

	
	// vytvorime druhou obrazovku
	back = create_bitmap(SCREEN_X, SCREEN_Y);
	if ( back == NULL ) {
		fprintf(stderr, "Nepodarilo se vytvorit druhou bitmapu.\n");
		exit(1);
	}
	
	
	// nacteme bitmaku
	ryba = load_bitmap( "ryba.bmp", NULL );
	if ( ryba == NULL ) {
		fprintf(stderr, "Nepodarilo se nacist soubor ryba.bmp.\n");
		exit(1);
	}

	// nastavime sourabnice ryby
	x = SCREEN_X/2;
	y = SCREEN_Y/2;

	// barva podkladu
	r = 0;
	g = 0;
	b = 0;
		
	// meneni bavy podkladu
	dr = 0.2;
	dg = 0.5;
	db = 0.7;
}


// Funkce main
int main( )
{
	// inicializace Allegra
	InitSystem();

	// priame klavesnici a mys
	install_keyboard();
	install_mouse();
	
	// nacteni bitmapek a podobne
	InitGame();
	
	
	while ( !key[KEY_Q] ) { // hrajeme, koduk se nezmackne q
		
		
		poll_keyboard(); // pred ctenim z klavesnice je vhodne volat
		
		// pohybova cast
		if ( key[KEY_LEFT] )  x--;
		if ( key[KEY_RIGHT] ) x++;
		if ( key[KEY_UP] )    y--;
		if ( key[KEY_DOWN] )  y++;
		
		// omezeni ryby
		if ( x < 0 ) x += SCREEN_X;
		if ( y < 0 ) y += SCREEN_Y;
		if ( y > SCREEN_Y ) y -= SCREEN_Y;
		if ( x > SCREEN_X ) x -= SCREEN_X;
		

		// neco malo s mysi
		poll_mouse();
	
	
		if (mouse_b & 1) { // kdyz se zmackne levy knoflik hodime rybu pod mys
			x = mouse_x;
			y = mouse_y;
		}
		
		
		// zmena barev :-)
		if ( (r += dr) > 255 ) r = 0;
		if ( (g += dg) > 255 ) g = 0;
		if ( (b += db) > 255 ) b = 0;
		
		// kreslici cast
		//  smazani obrazovky
		clear_to_color(back, makecol(r, g, b));
		

		// nakreslime rybu
		draw_sprite(back, ryba, x, y); 
		
		// aktualizace obrazovku
		draw_sprite(screen, back, 0, 0);
	}

	CleanAll();	

	return 0;
}
// tohle musi byt za funkci main - zalezitost allegra
END_OF_MAIN();

