/* allegro_6.c
 * Ukazka programu v Allegru 
 * prikladek pro pouzivani casu 
 *
 * Bernard Lidicky - 2004
 *
 * Je pod licenci GPL
 *
 * pro preklad mi fungovalo: "gcc allegro_6.c -o allegro_6 `allegro-config --cflags --libs`"
 */
 
#include <stdio.h>
#include <allegro.h>

// nastaveni rozliseni 
// V normalni hre by se melo nacotat z konfiguracniho souboru
#define SCREEN_X   640
#define SCREEN_Y   480
#define SCREEN_BPP  16


// druha bitmapa velikosti obrazovky
BITMAP *back = NULL;


// veci pro hru

// obrazek ryby
BITMAP *ryba;

// souradnice ryby
int x = 0, y = 0;

// souradnice ve float - v nich se pocita hra, v int se pak kresli
float fx = 0, fy = 0;

// zmena pohybu ryby
float dx = 0, dy = 0;

#define MAX_PARTIAL 1.0f
#define DD_SPEED    0.1f

// rychlost pohybu ryby
float speed = 4.0f;

#define MAX_SPEED 10.0f
#define D_SPEED   0.1f


// proveman pro mereni casu
volatile int time_var = 0;

/*
 * Funkce, ktera se bude volat kazdych 10ms
 */
void timer_callback()
{       
	// snizeni promene s casem o 1	
	if (time_var > 0) time_var--;
}
END_OF_FUNCTION(timer_callback);



/*
 * Provede inicializaci knihovny, nastavi rozliseni
 */
void InitSystem()
{
	

	// Inicializace Allegra
	if ( allegro_init() < 0 ) {
		fprintf(stderr, "Selhala inicializace Allegra.\n");
		exit(1);
	}

	// prirpavime klavesnici, mys a timer(cas)
	install_keyboard();
	install_timer();
	install_mouse();                                                                                                                    
		   
	// nastaveni barevne hloubky
	set_color_depth(SCREEN_BPP);
	
	// nastaveni grafickeho modu
//	if ( set_gfx_mode(GFX_AUTODETECT, SCREEN_X, SCREEN_Y, 0, 0) != 0 ){
	if ( set_gfx_mode(GFX_XWINDOWS, SCREEN_X, SCREEN_Y, 0, 0) != 0 ){
			fprintf(stderr, "Nepodarilo nastavit rozliseni %dx%dx%d\n", SCREEN_X, SCREEN_Y, SCREEN_BPP);
			exit(1);
	}


	// pripraveni funkci pro timer
	LOCK_VARIABLE(time_var);
	LOCK_FUNCTION(timer_callback);
	
	// nastaveni volani funkce timer_procedure kazdych 10ms
	install_int(timer_callback, 10);

	// vytvorime druhou obrazovku
	back = create_bitmap(SCREEN_X, SCREEN_Y);
	if ( back == NULL ) {
		fprintf(stderr, "Nepodarilo se vytvorit druhou bitmapu.\n");
		exit(1);
	}
	
	// zobrazime pointer mysi
	show_mouse(back);
}

/*
 * Provede na konci programu uklid
 */
void CleanAll()
{
	// uklidime mys
	show_mouse(NULL);

	// uvolneni nactene bitmapky
	destroy_bitmap(ryba);

	// uvolneni bitmapy s druhou obrazovkou
	destroy_bitmap(back);
	
	// ukonceni Allegra - neni nutne volat
	allegro_exit();
}

/*
 * Nacte bitmapy a podobne veci
 */
void InitGame()
{
	// nacteme bitmaku
	ryba = load_bitmap( "ryba.bmp", NULL );
	if ( ryba == NULL ) {
		fprintf(stderr, "Nepodarilo se nacist soubor ryba.bmp.\n");
		exit(1);
	}

	// nastavime sourabnice ryby
	fx = SCREEN_X/2;
	fy = SCREEN_Y/2;
}

/*
 * Funkce pro pohnuti svetem hry - tedy rybou a zpracovani vstupu uzivatele
 */
void MoveGameWorld()
{
	// aktualizace mysi a klavesnice
	poll_keyboard();
	poll_mouse();
	
	// zpena rychlosti pohybu 
	if ( key[KEY_UP] ) if ( (speed += D_SPEED) > MAX_SPEED) speed = MAX_SPEED;
	if ( key[KEY_DOWN] ) if ( (speed -= D_SPEED) < 0) speed = 0;
		

	// na stisk knofliku menime smer pohyby ryby 
	if (mouse_b & 1) {
		if (fx > mouse_x) if ( (dx -= DD_SPEED) < -MAX_PARTIAL) dx = -MAX_PARTIAL;
		if (fx < mouse_x) if ( (dx += DD_SPEED) >  MAX_PARTIAL) dx =  MAX_PARTIAL;
		if (fy > mouse_y) if ( (dy -= DD_SPEED) < -MAX_PARTIAL) dy = -MAX_PARTIAL;
		if (fy < mouse_y) if ( (dy += DD_SPEED) >  MAX_PARTIAL) dy =  MAX_PARTIAL;
	}

	// posunuti ryby
	fx += speed * dx;
	fy += speed * dy;

	
	// omezeni ryby
	if ( fx < 0 ) fx += SCREEN_X;
	if ( fy < 0 ) fy += SCREEN_Y;
	if ( fy > SCREEN_Y ) fy -= SCREEN_Y;
	if ( fx > SCREEN_X ) fx -= SCREEN_X;

	// konverze na int
	x = (int)fx;
	y = (int)fy;
	
}

/*
 * Funkce pro nakresleni sveta hry
 */
void DrawGameWorld()
{
	// schovame mys
	show_mouse(NULL);
	
	//  smazani obrazovky
	clear_to_color(back, makecol(80, 180, 250));

	// nakreslime rybu
	draw_sprite(back, ryba, x, y); 

	// nakreslime mys
	show_mouse(back);	
	
	// aktualizace obrazovku
	draw_sprite(screen, back, 0, 0);
}

// Funkce main
int main( )
{
	// inicializace Allegra
	InitSystem();

	// nacteni bitmapek a podobne
	InitGame();
	
	
	while ( !key[KEY_Q] ) { // hrajeme, koduk se nezmackne q
		
		// cekame, dokud timer nesnizi time_var na 0
		while ( time_var > 0 ) rest(5);
		
		// a opet ho nechame cekat
		time_var = 4;
		
		MoveGameWorld();

		DrawGameWorld();
		
	}

	CleanAll();	

return 0;
}
// tohle musi byt za funkci main - zalezitost allegra
END_OF_MAIN();

