/* sdl_4.c
 * Ukazka programu v SDL
 * Zobrazuje rybky a pouziva doublebuffering
 *
 * Bernard Lidicky - 2004
 *
 * Je pod licenci GPL
 *
 * pro preklad mi fungovalo: "gcc sdl_4.c -o sdl_4 `sdl-config --cflags --static-libs`"
 */

#include <stdio.h>
#include <SDL.h>

// nastaveni rozliseni 
// V normalni hre by se melo nacotat z konfiguracniho souboru
#define SCREEN_X   640
#define SCREEN_Y   480
#define SCREEN_BPP   0 // 0 == nezajima nas

SDL_Surface *screen; // obrazovka

SDL_Surface *ryba; // obrazek ryby
SDL_Rect   pozice; // pozice ryby
int         pohyb; // pro pohyb rybek

/*
 * Provede inicializaci knihovny, nastavi rozliseni
 */
void InitSystem()
{
	
	// Inicializace SDL a subsytemu s grafikou
	if ( SDL_Init(SDL_INIT_VIDEO|SDL_DOUBLEBUF) < 0 ) {
		fprintf(stderr, "Selhala inicializace SDL: %s\n", SDL_GetError());
		exit(1);
	}

	// Je vhodne pridat automaticke volani SDL_Quit pri ukonceni programu exitem
	atexit(SDL_Quit);

	// nastavime rozliseni okna a barevna hloubka nas nezajima
	if ( (screen = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, SCREEN_BPP, 0)) == NULL ) {
		fprintf(stderr, "Nepodarilo se vytvorit okno: %s\n", SDL_GetError());
		exit(2);
	}


}

/*
 * Provede na konci programu uklid
 */   
void CleanAll()
{
	// uvolnime obrazek ryby
	SDL_FreeSurface(ryba);

	
	// ukonceni SDL
	SDL_Quit();
}

/*
 * Nacte bitmapy a podobne veci
 */
void InitGame()
{

	// nacteme obrazek ryby
	ryba = SDL_LoadBMP("ryba.bmp");
	if (ryba == NULL) {
		fprintf(stderr, "Nepodarilo se nacist ryba.bmp: %s\n", SDL_GetError());
		exit(2);
	}

	// nastavime, ze ruzova se nekresli
	if ( SDL_SetColorKey(ryba,  SDL_SRCCOLORKEY, SDL_MapRGB( ryba->format, 255, 0, 255)) < 0 ) {
		fprintf(stderr, "Nepodarilo se nastavit colorkey: %s\n", SDL_GetError());
		exit(2);
	}

	// priprava rectu
	pozice.x = 0;
	pozice.y = 0;
	pozice.w = ryba->w;
	pozice.h = ryba->h;

	pohyb = 0;
	
}	

/*
 *  Zpracovana zpravy zasilane od SDL
 */
int EventLoop()
{
	SDL_Event event;
	
	// z fronty zprav vybirame zpravy, dokut tam nejake jsou
	while ( SDL_PollEvent(&event) ) {
		switch (event.type) {
   
		// zprava s pozadavkem na ukonceni aplikace
		case SDL_QUIT :  
			return 0;
			break;
		}
	}
	
	return 1;
}






// Funkce main
int main( )
{
        // inicializace SDL
	InitSystem();
	
	// nacteni bitmapek a podobne
	InitGame();

	
	
	// nejaka smycka zprav
	while ( EventLoop() ) {
	
		pohyb++;
	
		if ( pohyb > 480 ) break;	
		
		// vymazeme obrazovku do svetle modre
		SDL_FillRect(screen, NULL,  SDL_MapRGB( screen->format, 80, 180, 250));
		
		// nakreslime ryby
		pozice.x = pohyb;
		pozice.y = pohyb;
		SDL_BlitSurface(ryba, NULL, screen, &pozice);
		
		pozice.y = SCREEN_Y-pohyb;
		SDL_BlitSurface(ryba, NULL, screen, &pozice);
		
		pozice.y = SCREEN_Y/2;
		SDL_BlitSurface(ryba, NULL, screen, &pozice);
		
		pozice.x = SCREEN_Y/2;
		pozice.y = pohyb;
		SDL_BlitSurface(ryba, NULL, screen, &pozice);
		
		
		// aktualizujeme obrazovku
		SDL_Flip(screen);
	}
	return 0;
}
