/* sdl_6.c
 * Ukazka programu v SDL
 * Umoznuje ovladani rybky
 *
 * Bernard Lidicky - 2004
 *
 * Je pod licenci GPL
 *
 * pro preklad mi fungovalo: "gcc sdl_6.c -o sdl_6 `sdl-config --cflags --static-libs`"
 */

#include <stdio.h>
#include <SDL.h>

// nastaveni rozliseni 
// V normalni hre by se melo nacotat z konfiguracniho souboru
#define SCREEN_X   640
#define SCREEN_Y   480
#define SCREEN_BPP   0 // 0 == nezajima nas

SDL_Surface   *screen; // obrazovka
Uint8            *key; // buffer klavesnice
int           mouse_x; // mys x
int           mouse_y; // mys y



SDL_Surface *ryba; // obrazek ryby
SDL_Rect   pozice; // pozice ryby


// souradnice ve float - v nich se pocita hra, v int se pak kresli
float fx = 0, fy = 0;
//
// zmena pohybu ryby
float dx = 0, dy = 0;

#define MAX_PARTIAL 1.0f
#define DD_SPEED    0.1f

// rychlost pohybu ryby
float speed = 4.0f;

#define MAX_SPEED 10.0f
#define D_SPEED   0.1f


/*
 * Provede inicializaci knihovny, nastavi rozliseni
 */
void InitSystem()
{
	
	// Inicializace SDL a subsytemu s grafikou
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Selhala inicializace SDL: %s\n", SDL_GetError());
		exit(1);
	}

	// Je vhodne pridat automaticke volani SDL_Quit pri ukonceni programu exitem
	atexit(SDL_Quit);

	// nastavime rozliseni okna a barevna hloubka nas nezajima
	if ( (screen = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, SCREEN_BPP, 0)) == NULL ) {
		fprintf(stderr, "Nepodarilo se vytvorit okno: %s\n", SDL_GetError());
		exit(2);
	}


	// prvni cteni stavu klavesnice
	key = SDL_GetKeyState(NULL);
}

/*
 * Provede na konci programu uklid
 */   
void CleanAll()
{
	// uvolnime obrazek ryby
	SDL_FreeSurface(ryba);

	
	// ukonceni SDL
	SDL_Quit();
}

/*
 * Nacte bitmapy a podobne veci
 */
void InitGame()
{

	// nacteme obrazek ryby
	ryba = SDL_LoadBMP("ryba.bmp");
	if (ryba == NULL) {
		fprintf(stderr, "Nepodarilo se nacist ryba.bmp: %s\n", SDL_GetError());
		exit(2);
	}

	// nastavime, ze ruzova se nekresli
	if ( SDL_SetColorKey(ryba,  SDL_SRCCOLORKEY, SDL_MapRGB( ryba->format, 255, 0, 255)) < 0 ) {
		fprintf(stderr, "Nepodarilo se nastavit colorkey: %s\n", SDL_GetError());
		exit(2);
	}

	// priprava rectu
	pozice.x = SCREEN_X/2;
	pozice.y = SCREEN_Y/2;
	pozice.w = ryba->w;
	pozice.h = ryba->h;

	// priprava pozice
	fx = SCREEN_X/2;
	fy = SCREEN_Y/2;
	
}	


/*
 *  Zpracovana zpravy zasilane od SDL
 */
int EventLoop()
{
	SDL_Event event;
	
	// z fronty zprav vybirame zpravy, dokut tam nejake jsou
	while ( SDL_PollEvent(&event) ) {
		switch (event.type) {
   
		// zprava s pozadavkem na ukonceni aplikace
		case SDL_QUIT :  
			return 0;
			break;
		}
	}
	
	return 1;
}



/*
 * Funkce pro pohnuti svetem hry - tedy rybou a zpracovani vstupu uzivatele
 */
void MoveGameWorld(int dtime)
{
	// pro prepocitani casu
	float time_scale = (float)dtime / 40.0f;
	
	// aktulizace klavesnice
	key = SDL_GetKeyState(NULL);
	 	
	// zpena rychlosti pohybu 
	if ( key[SDLK_UP] )   if ( (speed += D_SPEED*time_scale) > MAX_SPEED) speed = MAX_SPEED;
	if ( key[SDLK_DOWN] ) if ( (speed -= D_SPEED*time_scale) < 0) speed = 0;
		

	// pohyb pres mys
	if ( SDL_GetMouseState(&mouse_x, &mouse_y) & SDL_BUTTON_LEFT ) {
		if (fx > mouse_x) if ( (dx -= DD_SPEED*time_scale) < -MAX_PARTIAL) dx = -MAX_PARTIAL;
		if (fx < mouse_x) if ( (dx += DD_SPEED*time_scale) >  MAX_PARTIAL) dx =  MAX_PARTIAL;
		if (fy > mouse_y) if ( (dy -= DD_SPEED*time_scale) < -MAX_PARTIAL) dy = -MAX_PARTIAL;
		if (fy < mouse_y) if ( (dy += DD_SPEED*time_scale) >  MAX_PARTIAL) dy =  MAX_PARTIAL;
	}

	// posunuti ryby
	fx += (speed * dx) * time_scale;
	fy += (speed * dy) * time_scale;

	
	// omezeni ryby
	if ( fx < 0 ) fx += SCREEN_X;
	if ( fy < 0 ) fy += SCREEN_Y;
	if ( fy > SCREEN_Y ) fy -= SCREEN_Y;
	if ( fx > SCREEN_X ) fx -= SCREEN_X;

	// konverze do pozice
	pozice.x = (int)fx;
	pozice.y = (int)fy;
}

/*
 * Funkce pro nakresleni sveta hry
 */
void DrawGameWorld()
{
	// vymazeme obrazovku
	SDL_FillRect(screen, NULL,  SDL_MapRGB( screen->format, 80, 180, 250));
		
	// nakreslime rybu
	SDL_BlitSurface(ryba, NULL, screen, &pozice);
		
		
	// aktualizujeme obrazovku - 4 nuly == vsechno
	SDL_UpdateRect(screen, 0, 0, 0, 0);
}

// Funkce main
int main( )
{
	int time1, time2; // mereni casu
	int dtime;        // rozdil casu
	
        // inicializace SDL
	InitSystem();
	
	// nacteni bitmapek a podobne
	InitGame();

	time1 = SDL_GetTicks();
	time2 = SDL_GetTicks();
	
	                 	
	// nejaka smycka zprav
	while ( EventLoop() &&  (key[SDLK_q] == 0)) {
	
		// zjistime novy cas
		time1 = SDL_GetTicks();
	
		// pokud se zmenil, provedeme prekresleni
		if ( time1 - time2 > 0 ) {
		
			// zjistime rozdil casu
			if ( (dtime = time1-time2) > 100) dtime = 100;
			
			// ulozime cas pozledni aktualizace
			time2 = time1;
			
			MoveGameWorld(dtime);
	
			DrawGameWorld();
			
		}
		
	}
	return 0;
}
